# Artifact Appendix
This document describes the requirements, expected outputs, and usage instructions of the software artifact that accompanies the paper "Optimal Graph Stretching for Distributed Averaging".
This document was modelled after the [Artifact Evaluation file used at the Privacy Enhancing Technologies 2025 symposium](https://petsymposium.org/artifacts.php), though the paper that this artifact accompanies has not been published there.


## Description
Generates random graphs, increases their girth, heuristically optimises the topology, and measures performance of distributed averaging.

### Security/Privacy Issues and Ethical Concerns
No issues or concerns. All code consists of numerical simulations on artificial datasets.


## Basic Requirements
### Hardware Requirements
Very high CPU compute power. More cores is better.

### Software Requirements
Code is written in MATLAB. This is a proprietary tool that requires a license. The code is not compatible with GNU Octave.

The following packages are required:
* MATLAB R2021b or newer
* MATLAB Parallel Computing Toolbox
* MATLAB Statistics and Machine Learning Toolbox

You can download MATLAB at <https://mathworks.com/downloads/>. For more information on installing MATLAB and its toolboxes, see <https://mathworks.com/help/install/ug/install-products-with-internet-connection.html>.

### Estimated Time and Storage Consumption

| **Physical cores** | 64       | 1280     |
|--------------------|----------|----------|
| **Time**           | 92 hours | 13 hours |
| **Memory**         | 75 GB    | 1362 GB  |
| **Storage**        | 2.7 GB   | 2.7 GB   |

> **Physical cores and logical cores**  
> Note the distinction between physical cores and logical cores. On a typical personal computer, you have two logical cores per physical core. If your computer says it has 16 logical cores, you likely have only 8 physical cores.

> **How many physical cores do I have?**  
> The number of physical cores can typically be found in your operating system's settings, task manager, or in some sort of "About" section. Search for `number of cores [windows / macos / kde / gnome / ...]` in your favourite search engine for more information.

> **Reducing execution time and memory usage**  
> You can choose to reduce execution time and memory usage at the cost of reduced output accuracy by appropriately adjusting the experiments' input parameters. Relevant instructions are provided in the "Experiments" section.


## Environment
### Accessibility
All code is available at <https://doi.org/10.4121/e64c61d3-deb5-4aad-af60-92d92755781f>. Clone the git repository linked on that page.

The distribution artifact (ending in `.cache.7z`) is not required. Download it if you are interested in inspecting the experiment's outputs without reproducing them yourself. Relevant instructions are provided in the "Experiments" section.

### Set up the environment
1. (Optional: Inspect Only) Extract the distribution artifact `StretchSim.cache.7z` into the git repository. Ensure that the `cache` directory is in the same directory as `StretchSim.m` and is populated with cache files. That is, you should have the file `StretchSim/StretchSim.m`, and the directory `StretchSim/cache/a/a/` should contain many `.mat` files.
2. Launch MATLAB.
3. Install the "Parallel Computing Toolbox" by MathWorks:
   1. At the top of the MATLAB window, click the "Home" tab, and click "Add-Ons". The "Add-On Explorer" will open in a new window.
   2. Search for "Parallel Computing Toolbox" and go to the toolbox's page.
   3. Confirm that the toolbox is published by MathWorks.
   4. Click "Install" and then click "Continue". MATLAB will close and an installer will open.
   5. Walk through the steps of the installer. After completion, the installer will close and MATLAB will open.
4. Similarly, install the "Statistics and Machine Learning Toolbox" by MathWorks.
5. Install the "Progress bar (cli, gui, parfor)" add-on (v1.3.1) by HyunGwang Cho:
   1. Open the "Add-On Explorer" as described above.
   2. Search for "Progress bar (cli, gui, parfor)" and go to the add-on's page.
   3. Confirm that the add-on is published by Armin Ataei.
   4. Click "Add" and then "Add to MATLAB" to install the add-on.
6. Similarly, install the "matlab2tikz/matlab2tikz" add-on (v1.50.0.0) by Nico Schlömer.
7. Close the add-on explorer.

### Testing the Environment
To test the environment, you will need the "Command Window" (typically in the bottom right) to execute a few commands.

1. Run the command `parpool(2)`. A parallel pool should be started using the 'Processes' profile. Wait for the pool to have started. Shut down the parallel pool again by running the command `delete(gcp('nocreate'))`. If you do not get any errors, the "Parallel Computing Toolbox" was installed correctly.
2. Run the command `randsample(10, 1)`. If you do not get any errors, the "Statistics and Machine Learning Toolbox" was installed correctly.
3. Run the command `p = ProgressBar(1)`. A progress bar window should appear. Close the window with `delete(p)`. If you do not get any errors, the add-on "Progress bar (cli, gui, parfor)" was installed correctly.
4. Run the command `matlab2tikz('figurehandle', figure)`. An empty figure and a file selector should appear. You can close both manually. If you do not get any errors beyond "Invalid path. The path must not contain a null character.", the add-on "matlab2tikz/matlab2tikz" was installed correctly.


## Artifact Evaluation
### Main Results and Claims
#### Main Result 1: StretchSim
(The following is an excerpt from the paper.)

Stretching the girth of a graph increases convergence time proportional to the number of edges removed.
Consequently, stretching by iteratively removing the edge that is simultaneously in the largest number of cycles results in the smallest convergence time cost.
Furthermore, convergence time can be recuperated using a greedy algorithm to add edges without decreasing girth. 
Finally, minimising the number of leaves does not affect convergence time.

See Section 6 of the paper, including Figures 1 through 8.

### Experiments
> **"array exceeds maximum array size preference" error**  
> If you receive this error at any point, your machine does not have sufficient memory available to store the necessary data structures. You can resolve this error in three ways:
> 1. Acquire more RAM, either by closing other programs, or by adding more RAM to your machine.
> 2. Allow MATLAB to temporarily store data on disk. To do so, find the "Home" tab at the top of MATLAB, click "Preferences", go to "Workspace", and disable the option "Limit the maximum array size to a percentage of RAM". Note, however, that doing so is likely to significantly slow down the experiments, possibly by several orders of magnitude.

#### Experiment 1: StretchSim
* **Expected result:** Exact replica of Figures 1 through 8.
* **Claim:** See Main Result 1.
* **Requirements:**

  | **Physical cores** | 64       | 1280     |
  |--------------------|----------|----------|
  | **Time**           | 92 hours | 13 hours |
  | **Memory**         | 75 GB    | 1362 GB  |
  | **Storage**        | 2.7 GB   | 2.7 GB   |

1. Set up the environment as described above.
2. In MATLAB, navigate to the directory `StretchSim/`.
3. By default, cached results are loaded if they are present. Therefore:
   * (Default) To reproduce the results yourself, ensure that the directory `StretchSim/cache/` is empty or does not exist before your first run. You will see that during step 7 below, the directory `StretchSim/cache/` will be filled again; this is fine.
   * (Optional) Alternatively, to analyse the generated results without reproducing them yourself, make sure that distribution artifact `StretchSim.cache.7z` has been extracted properly, as described in the section "Environment".
4. Open the file `StretchSim.m` in MATLAB.
5. (Optional) To limit the number of physical cores used in the experiment, find `parallel_max_workers = Inf` and change `Inf` to the desired value. A value of `Inf` indicates that all physical cores on your machine should be used.
6. (Optional) To reduce execution time and memory usage at the cost of accuracy, find `repeat_count = 1000` and change `1000` to a lower value. Note that doing so will result in figures that are not exactly the same as in the paper.
7. Run `StretchSim.m`:
    * (Method 1, recommended) To run the script on a machine with [SLURM](https://slurm.schedmd.com/), you can use the files `SlurmStretchSim.sbatch` and `SlurmStretchFig.sbatch` as a starting point. The configuration provided there was written with the [Delft AI Cluster](https://daic.tudelft.nl/) in mind, but can easily be adapted for other settings. After changing the files appropriately, run `sbatch SlurmStretchSim.sbatch` to queue running the computations. Once that has completed successfully, run `sbatch SlurmStretchFig.sbatch` to queue compiling the figures.
    * (Method 2) With `StretchSim.m` opened in the editor, click the "Editor" tab at the top, and click "Run".
8. Keep track of progress and check for errors in the "Command Window". If all goes correctly, a separate progress bar window appears, which will close again once all experiments have completed running. Wait for the post-processing to complete. The program is done when the message "StretchSim has completed running." is written to the "Command Window".

The output of each individual experiment is stored in the `cache/` directory. You can safely stop and resume `StretchSim` at any point without losing much progress.

The created figures are stored in the `figs/` directory, unless configured otherwise.


## Limitations
None.


## Notes on Reusability
The code is documented, well-structured, and extensible. The code could easily be changed to consider different types of graphs, stretching algorithms, distributed protocols, metrics, and visualisations.
